@@ -21,7 +21,7 @@ from reportlab.pdfbase import pdfdoc
 from reportlab.pdfbase import pdfmetrics
 from reportlab.pdfbase.ttfonts import ShapedStr, shapeFragWord
 from reportlab.pdfgen  import pathobject
-from reportlab.pdfgen.textobject import PDFTextObject, _PDFColorSetter
+from reportlab.pdfgen.textobject import PDFTextObject, _PDFColorSetter, bidiShapedText
 from reportlab.lib.colors import black, _chooseEnforceColorSpace, Color, CMYKColor, toColor
 from reportlab.lib.utils import ImageReader, isSeq, isStr, isUnicode, _digester, asUnicode
 from reportlab.lib.abag import ABag
@@ -1609,19 +1609,9 @@ class Canvas(_PDFColorSetter):
         # use PDFTextObject for multi-line text.
         ##################################################
 
-    def shapedText(self,text):
-        if not isinstance(text,ShapedStr):
-            text = asUnicode(text)
-            font = pdfmetrics.getFont(self._fontname)
-            if font.isShaped:
-                text = shapeFragWord([0,(ABag(fontName=self._fontname,fontSize=self._fontsize),text)])[1][1]
-        width = (sum((_.x_advance for _ in text.__shapeData__))*self._fontsize/1000 if isinstance(text,ShapedStr)
-                    else self.stringWidth(text, self._fontname, self._fontsize))
-        return text, width
-
-    def drawString(self, x, y, text, mode=None, charSpace=0, direction=None, wordSpace=None):
+    def drawString(self, x, y, text, mode=None, charSpace=0, direction=None, wordSpace=None, shaping=False):
         """Draws a string in the current text styles."""
-        text, width = self.shapedText(text)
+        text, width = bidiShapedText(text,direction,fontName=self._fontname,fontSize=self._fontsize, shaping=shaping)
         #we could inline this for speed if needed
         t = self.beginText(x, y, direction=direction)
         if mode is not None: t.setTextRenderMode(mode)
@@ -1633,12 +1623,12 @@ class Canvas(_PDFColorSetter):
         if mode is not None: t.setTextRenderMode(0)
         self.drawText(t)
 
-    def drawRightString(self, x, y, text, mode=None, charSpace=0, direction=None, wordSpace=None):
+    def drawRightString(self, x, y, text, mode=None, charSpace=0, direction=None, wordSpace=None, shaping=False):
         """Draws a string right-aligned with the x coordinate"""
-        text, width = self.shapedText(text)
+        text, width = bidiShapedText(text,direction,fontName=self._fontname,fontSize=self._fontsize, shaping=shaping)
         if charSpace: width += (len(text)-1)*charSpace
         if wordSpace: width += (text.count(u' ')+text.count(u'\xa0')-1)*wordSpace
-        t = self.beginText(x - width, y, direction=direction)
+        t = self.beginText(x - width, y)
         if mode is not None: t.setTextRenderMode(mode)
         if charSpace: t.setCharSpace(charSpace)
         if wordSpace: t.setWordSpace(wordSpace)
@@ -1648,14 +1638,14 @@ class Canvas(_PDFColorSetter):
         if mode is not None: t.setTextRenderMode(0)
         self.drawText(t)
 
-    def drawCentredString(self, x, y, text, mode=None, charSpace=0, direction=None, wordSpace=None):
+    def drawCentredString(self, x, y, text, mode=None, charSpace=0, direction=None, wordSpace=None, shaping=False):
         """Draws a string centred on the x coordinate. 
         
         We're British, dammit, and proud of our spelling!"""
-        text, width = self.shapedText(text)
+        text, width = bidiShapedText(text,direction,fontName=self._fontname,fontSize=self._fontsize, shaping=shaping)
         if charSpace: width += (len(text)-1)*charSpace
         if wordSpace: width += (text.count(u' ')+text.count(u'\xa0')-1)*wordSpace
-        t = self.beginText(x - 0.5*width, y, direction=direction)
+        t = self.beginText(x - 0.5*width, y)
         if mode is not None: t.setTextRenderMode(mode)
         if charSpace: t.setCharSpace(charSpace)
         if wordSpace: t.setWordSpace(wordSpace)
@@ -1665,7 +1655,7 @@ class Canvas(_PDFColorSetter):
         if mode is not None: t.setTextRenderMode(0)
         self.drawText(t)
 
-    def drawAlignedString(self, x, y, text, pivotChar=rl_config.decimalSymbol, mode=None, charSpace=0, direction=None, wordSpace=None):
+    def drawAlignedString(self, x, y, text, pivotChar=rl_config.decimalSymbol, mode=None, charSpace=0, direction=None, wordSpace=None, shaping=False):
         """Draws a string aligned on the first '.' (or other pivot character).
 
         The centre position of the pivot character will be used as x.
@@ -1707,19 +1697,19 @@ class Canvas(_PDFColorSetter):
                 leftText = leftText[0:-1]
 
             self.drawRightString(x-0.5*pivW, y, leftText, mode=mode, charSpace=charSpace,
-                    direction=direction, wordSpace=wordSpace)
+                    direction=direction, wordSpace=wordSpace, shaping=shaping)
             self.drawString(x-0.5*pivW, y, rightText, mode=mode, charSpace=charSpace,
-                    direction=direction, wordSpace=wordSpace)
+                    direction=direction, wordSpace=wordSpace, shaping=shaping)
 
         else:
             #normal case
             leftText = parts[0]
             self.drawRightString(x-0.5*pivW, y, leftText, mode=mode, charSpace=charSpace,
-                    direction=direction, wordSpace=wordSpace)
+                    direction=direction, wordSpace=wordSpace, shaping=shaping)
             if len(parts) > 1:
                 rightText = pivotChar + parts[1]
                 self.drawString(x-0.5*pivW, y, rightText, mode=mode, charSpace=charSpace,
-                        direction=direction, wordSpace=wordSpace)
+                        direction=direction, wordSpace=wordSpace, shaping=shaping)
 
     def getAvailableFonts(self):
         """Returns the list of PostScript font names available.
